from PIL import Image
import matplotlib.pyplot as plot
import drawsvg as dw
import subprocess

# Assume all images are of a same size.
def tile(*images, vertical=False):
    width, height = images[0].width, images[0].height
    tiled_size = (
        (width, height * len(images))
        if vertical
        else (width * len(images), height)
    )
    tiled_img = Image.new(images[0].mode, tiled_size)
    row, col = 0, 0
    for image in images:
        tiled_img.paste(image, (row, col))
        if vertical:
            col += height
        else:
            row += width
    return tiled_img

def describe( img ):
    print( "Image: ", img.format, img.size, img.mode )

def threshold( img, n ):
    return img.point( lambda x: 255 if x>n else 0)

def rgb_brightness( img, n ):
    pixels = img.load()
    for i in range(img.width):
        for j in range(img.height):
            r, g, b = pixels[i,j]
            r += n
            g += n
            b += n
            pixels[i,j] = (r,g,b)

def rgb_contrast( img, f ):
    pixels = img.load()
    for i in range(img.width):
        for j in range(img.height):
            r, g, b = pixels[i,j]
            r = int( r * f )
            g = int( g * f )
            b = int( b * f )
            pixels[i,j] = (r,g,b)

def rgb_contrast_brightness( img, factor, bias ):
    img1 = img.copy()
    pixels = img1.load()
    for i in range(img1.width):
        for j in range(img1.height):
            r, g, b = pixels[i,j]
            r = int( r * factor + bias )
            g = int( g * factor + bias )
            b = int( b * factor + bias )
            pixels[i,j] = (r,g,b)
    return img1

def contrast_brightness( img, c, b ):
    return img.point( lambda x: x * c + b )

def img_range( img, min, max ):
    return contrast_brightness( img, 255.0/(max-min), -min * 255.0/(max-min))

def plot_rgb_histogram( img ):
    h = img.histogram()
    plot.bar( range(0,256), h[0:256], color="#ff0000", alpha=0.5 )
    plot.bar( range(0,256), h[256:512], color="#00aa00", alpha=0.5 )
    plot.bar( range(0,256), h[512:768], color="#0000ff", alpha=0.5 )
    plot.show()

def write_svg_to_file( d, filename ):
    svgfile = open( filename, "w" )
    svgfile.write( d.as_svg() )
    svgfile.close()

def inkscape_convert_svg_to_png( svgfilename, pngfilename ):
    cmd_list = ['C:\\Program Files\\Inkscape\\bin\\inkscape.exe', 
                '--export-filename=' + pngfilename,
                svgfilename ]
    p = subprocess.run( cmd_list, capture_output=True )
    if len( p.stdout ) > 0:
        print( p.stdout.decode('utf-8') )
    if len( p.stderr ) > 0 or p.returncode:
        raise Exception( 'Inkscape error [' + str(p.returncode) + ']:\n' + p.stderr.decode('utf-8') )
